// Package json_tool ...
//
// Description : json_tool ...
//
// Author : go_developer@163.com<张德满>
//
// Date : 2022/01/22 9:19 PM
package json_tool

import (
	"strings"

	"github.com/tidwall/gjson"

	"github.com/Jeffail/gabs"
)

// FilterDataRule 参数过滤规则
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2022/1/22 9:44 PM
type FilterDataRule struct {
	SourceKey    string      // 原始数据路径
	MapKey       string      // 提取后映射到的数据路径
	DefaultValue interface{} // 原始数据路径不存在时的默认值
	WithDefault  bool        // 是否使用默认值
}

// NewDataFilter 获取数据过滤方法实例
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2022/1/22 9:50 PM
func NewDataFilter(source string, filterRule []FilterDataRule) *DataFilter {
	return &DataFilter{
		source:     source,
		filterRule: filterRule,
	}
}

// DataFilter 数据过滤
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2022/1/22 9:20 PM
type DataFilter struct {
	source     string
	filterRule []FilterDataRule
}

// Filter 数据过滤
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2022/1/22 9:36 PM
func (df *DataFilter) Filter() (string, error) {
	var (
		jsonObject *gabs.Container
		err        error
	)
	// 创建数据的根结点
	jsonObject = gabs.New()
	for _, item := range df.filterRule {
		if strings.Contains(item.SourceKey, "[]") {
			// 数组, 特殊处理
			continue
		}
		sourceSearchResult := gjson.Get(df.source, item.SourceKey)
		if !sourceSearchResult.Exists() {
			if item.WithDefault {
				if _, err = jsonObject.SetP(item.DefaultValue, item.MapKey); nil != err {
					return "", err
				}
			}
			continue
		}
		if _, err = jsonObject.SetP(sourceSearchResult.Value(), item.MapKey); nil != err {
			return "", err
		}
	}
	return jsonObject.String(), nil
}
